###################################################
### chunk number 1: preliminaries
###################################################
options(width=80, show.signif.stars = FALSE,
        lattice.theme = function() canonical.theme("pdf", color = FALSE))
library(Matrix)
#library(MASS)
library(lattice)
library(lme4)
library(mlmRev)
#library(coda)
set.seed(1234321)


###################################################
### chunk number 2: Examprep
###################################################
lmer(normexam ~ standLRT + sex + schgend + (1|school), Exam)


###################################################
### chunk number 3: ExamData
###################################################
str(Exam)
summary(Exam)


###################################################
### chunk number 4: ExamFit
###################################################
(Em1 <- lmer(normexam ~ standLRT + sex + schgend + (1|school), Exam))


###################################################
### chunk number 5: Examtime
###################################################
system.time(lmer(normexam ~ standLRT + sex + schgend + (1|school), Exam))


###################################################
### chunk number 6: ExamRelevel
###################################################
Exam$sex     <- relevel(Exam$sex, "M")
Exam$schgend <- relevel(Exam$schgend, "girls")
(Em2 <- lmer(normexam ~ standLRT + sex + schgend + (1|school), Exam))


###################################################
### chunk number 7: ExamIds
###################################################
Exam$ids <- with(Exam, school:student)[, drop = TRUE]
str(Exam)


###################################################
### chunk number 8: dupExamIds
###################################################
as.character(Exam$ids[which(duplicated(Exam$ids))])


###################################################
### chunk number 9: OnlyBoy
###################################################
subset(Exam, ids == '43:86')
xtabs(~ sex + school, Exam, subset = school %in% c(43, 50, 52), drop = TRUE)


###################################################
### chunk number 10: ExamXtabs
###################################################
xtabs(~ sex + school, Exam, subset = type == "Mxd", drop = TRUE)


###################################################
### chunk number 11: ExamMosaicshow eval=FALSE
###################################################
## ExamMxd <- subset(Exam, type == "Mxd")
## ExamMxd$school <- ExamMxd$school[,drop = TRUE]
## mosaicplot(~ school + sex, ExamMxd)


###################################################
### chunk number 12: ExamMosaic
###################################################
ExamMxd <- subset(Exam, type == "Mxd")
ExamMxd$school <- ExamMxd$school[,drop = TRUE]
mosaicplot(~ school + sex, ExamMxd)


###################################################
### chunk number 13: Examplot1
###################################################
print(xyplot(normexam ~ standLRT | sex * type, Exam,
             type = c("g", "p", "smooth"), layout = c(2,2),
             xlab = "Standardized London Reading Test score",
             ylab = "Normalized exam score", aspect = 1.2))


###################################################
### chunk number 14: Examplot1show eval=FALSE
###################################################
## xyplot(normexam ~ standLRT | sex * type, Exam, type = c("g", "p", "smooth"))


###################################################
### chunk number 15: Examplot2
###################################################
print(xyplot(normexam ~ standLRT, Exam, groups = sex:type,
             type = c("g", "smooth"), xlim = c(-3,3), ylim = c(-2,2),
             xlab = "Standardized London Reading Test score",
             ylab = "Normalized exam score",
             auto.key = list(space = 'right', points = FALSE, lines = TRUE),
             aspect = 1))


###################################################
### chunk number 16: Examplot2show eval=FALSE
###################################################
## xyplot(normexam ~ standLRT, Exam, groups = sex:type, type = c("g", "smooth"))


###################################################
### chunk number 17: Examplot3
###################################################
print(xyplot(normexam ~ standLRT | school, Exam,
             type = c("g", "p", "r"),
             xlab = "Standardized London Reading Test score",
             ylab = "Normalized exam score",
             subset = sex == "F" & type == "Sngl"))


###################################################
### chunk number 18: Examplot3show
###################################################
xyplot(normexam ~ standLRT | school, Exam,
             type = c("g", "p", "r"),
             subset = sex == "F" & type == "Sngl")


###################################################
### chunk number 19: Examplot4show eval=FALSE
###################################################
## xyplot(normexam ~ standLRT | school, Exam, type = c("g", "p", "r"),
##              subset = sex == "F" & type == "Sngl" & school != 48,
##              index.cond = function(x, y) coef(lm(y ~ x))[1])


###################################################
### chunk number 20: Examplot4
###################################################
print(xyplot(normexam ~ standLRT | school, Exam,
             type = c("g", "p", "r"),
             xlab = "Standardized London Reading Test score",
             ylab = "Normalized exam score",
             subset = sex == "F" & type == "Sngl" & school != 48,
             index.cond = function(x, y) coef(lm(y ~ x))[1]))


###################################################
### chunk number 21: Examplot4a
###################################################
print(xyplot(normexam ~ standLRT | school, Exam,
             type = c("g", "p", "r"),
             xlab = "Standardized London Reading Test score",
             ylab = "Normalized exam score",
             subset = sex == "F" & type == "Sngl" & school != 48,
             index.cond = function(x, y) coef(lm(y ~ x))[2]))


###################################################
### chunk number 22: ExamLmListFS
###################################################
show(ExamFS <- lmList(normexam ~ standLRT | school, Exam,
                      subset = sex == "F" & type == "Sngl" & school != 48))


###################################################
### chunk number 23: Examplot4cshow
###################################################
plot(confint(ExamFS, pool = TRUE), order = 1)


###################################################
### chunk number 24: Examplot4c
###################################################
print(plot(confint(ExamFS, pool = TRUE), order = 1))


###################################################
### chunk number 25: Examplot5
###################################################
print(xyplot(normexam ~ standLRT | school, Exam,
             type = c("g", "p", "r"),
             xlab = "Standardized London Reading Test score",
             ylab = "Normalized exam score",
             subset = sex == "M" & type == "Sngl", layout = c(5,2),
             index.cond = function(x, y) coef(lm(y ~ x))[1]))


###################################################
### chunk number 26: ExamLmListMS
###################################################
show(ExamMS <- lmList(normexam ~ standLRT | school, Exam,
                      subset = sex == "M" & type == "Sngl"))


###################################################
### chunk number 27: Examplot5b
###################################################
print(plot(confint(ExamMS, pool = TRUE), order = 1))


###################################################
### chunk number 28: Examplot6
###################################################
print(xyplot(normexam ~ standLRT | school, Exam, groups = sex,
             type = c("g", "p", "r"),
             xlab = "Standardized London Reading Test score",
             ylab = "Normalized exam score",
             subset = !school %in% c(43, 47) & type == "Mxd",
             index.cond = function(x, y) coef(lm(y ~ x))[1],
             auto.key = list(space = 'top', lines = TRUE,
             columns = 2), layout = c(7,5),
             aspect = 1.2))


###################################################
### chunk number 29: ExamLmListM
###################################################
show(ExamM <- lmList(normexam ~ standLRT + sex| school, Exam,
                     subset = type == "Mxd" & !school %in% c(43,47,54)))


###################################################
### chunk number 30: Examplot6b
###################################################
print(plot(confint(ExamM, pool = TRUE), order = 1))


###################################################
### chunk number 31: Em3
###################################################
(Em3 <- lmer(normexam ~ standLRT + sex + type + (1|school), Exam))


###################################################
### chunk number 32: Em4
###################################################
(Em4 <- lmer(normexam ~ standLRT + sex + type + (standLRT|school), Exam))


###################################################
### chunk number 33: EmAnova
###################################################
anova(Em3, Em4)


###################################################
### chunk number 34: Em5
###################################################
(Em5 <- lmer(normexam ~ standLRT + sex + type + (standLRT + sex|school), Exam))


###################################################
### chunk number 35: Es4
###################################################
#Es4 <- mcmcsamp(Em4, 10000, trans = FALSE)


###################################################
### chunk number 36: Examplot7
###################################################
#print(densityplot(Es4, plot.points = FALSE, ylab = "",
#                  scales = list(x = list(axs = "i"))))


###################################################
### chunk number 37: Examplot8
###################################################
print(qqmath(Es4, ylab = "",
             xlab = "Quantiles of a standard normal distribution",
             scales = list(y = list(relation = "free"),
             x = list(axs = "i")), type = c("g", "p"),
             pch = ".", cex = 1.5))


###################################################
### chunk number 38: Examplot9
###################################################
Es4a <- Es4[,5:8]
Es4a[,4] <- atanh(Es4a[,4]/sqrt(Es4a[,2]*Es4a[,3]))
for (i in 1:3) Es4a[,i] <- log(Es4a[,i])
print(densityplot(Es4a, plot.points = FALSE, ylab = "",
                  scales = list(x = list(axs = "i"))))


###################################################
### chunk number 39: Examplot10
###################################################
print(qqmath(Es4a,
             ylab = "",
             xlab = "Quantiles of a standard normal distribution",
             layout = c(2,2),
             scales = list(y = list(relation = "free"),
             x = list(axs = "i")), type = c("g", "p"), pch = ".",
             cex = 1.5))


###################################################
### chunk number 40: Chem97prep
###################################################
lmer(score ~ gcsecnt + (1|school) + (1|lea), Chem97)


###################################################
### chunk number 41: Chem97
###################################################
str(Chem97)
system.time(mC1 <- lmer(score ~ 1+(1|lea:school) + (1|lea), Chem97))
summary(mC1)
system.time(mC2 <- lmer(score ~ gcsecnt + (1|school) + (1|lea), Chem97,
                        control = list(niterEM = 40)), gc = TRUE)
summary(mC2)


###################################################
### chunk number 42: Contraception
###################################################
str(Contraception)


###################################################
### chunk number 43: Contraceptionprep
###################################################
lmer(use ~ urban+age+livch+(1|district), Contraception, binomial)


###################################################
### chunk number 44: Contraception
###################################################
system.time(Cm1 <- lmer(use ~ age+urban+livch+(1|district),
                        Contraception, binomial))
Cm1
system.time(Cm2 <- lmer(use ~ age+urban+livch+(1|district),
                        Contraception, binomial,
                        method = "Laplace"))
Cm2
## system.time(Cm3 <- lmer(use ~ age+urban+livch+(1|district),
##                         Contraception, binomial,
##                         method = "AGQ"))
## Cm3
system.time(Cm4 <- lmer(use ~ age+urban+livch+(urban|district),
                        Contraception, binomial))
Cm4
system.time(Cm5 <- lmer(use ~ age+urban+livch+(urban|district),
                        Contraception, binomial,
                        method = "Laplace"))
Cm5


###################################################
### chunk number 45: Contra1
###################################################
print(xyplot(ifelse(use == "Y", 1, 0) ~ age|urban, Contraception,
             groups = livch, type = c("g", "smooth"),
             auto.key = list(space = "top", points = FALSE,
             lines = TRUE, columns = 4),
             ylab = "Proportion", xlab = "Centered age"))


###################################################
### chunk number 46: Cm6
###################################################
(Cm6 <- lmer(use ~ age + I(age^2) + urban + livch + (1|district),
             Contraception, binomial, method = "Laplace"))
anova(Cm1, Cm6)


###################################################
### chunk number 47: children
###################################################
Contraception$children <- factor(ifelse(Contraception$livch == 0, "N", "Y"))
(Cm7 <- lmer(use ~ age + I(age^2) + urban + children + (1|district),
             Contraception, binomial, method = "Laplace"))
anova(Cm6, Cm7)


###################################################
### chunk number 48: Cm8
###################################################
(Cm8 <- lmer(use ~ age + I(age^2) + urban * children + (1|district),
             Contraception, binomial, method = "Laplace"))


###################################################
### chunk number 49: Cm9
###################################################
(Cm9 <- lmer(use ~ age + I(age^2) + urban + children + (urban|district),
             Contraception, binomial, method = "Laplace"))
anova(Cm7, Cm9)
(Cm10 <- lmer(use ~ age + I(age^2) + urban + children + (children|district),
             Contraception, binomial, method = "Laplace"))
anova(Cm7, Cm10)


###################################################
### chunk number 50: Cm11
###################################################
##(Cm11 <- lmer(use ~ age + I(age^2) + urban + children + (1|district), Contraception, binomial, method = "Laplace"))
#Cs11 <- mcmcsamp(Em4, 10000, trans = FALSE) # mcmcsamp for GLMMs in supernodal representation not yet implemented


###################################################
### chunk number 51: Contra2
###################################################
#print(densityplot(Cs11, plot.points = FALSE, ylab = "",
#                  scales = list(x = list(axs = "i"))))


###################################################
### chunk number 52: Contra3
###################################################
#print(qqmath(Cs11,
#             ylab = "",
#             xlab = "Quantiles of a standard normal distribution",
#             scales = list(y = list(relation = "free"),
#             x = list(axs = "i")), type = c("g", "p"),
#             pch = ".", cex = 1.5))


###################################################
### chunk number 53: Oxboys
###################################################
str(Oxboys)
system.time(mX1 <- lmer(height ~ age + I(age^2) + I(age^3) + I(age^4) + (age + I(age^2)|Subject),
                       Oxboys))
summary(mX1)
system.time(mX2 <- lmer(height ~ poly(age,4) + (age + I(age^2)|Subject), Oxboys))
summary(mX2)


###################################################
### chunk number 54: ScotsSec
###################################################
str(ScotsSec)
system.time(mS1 <- lmer(attain ~ sex + (1|primary) + (1|second), ScotsSec), gc=TRUE)
summary(mS1)


